class SaleChallenge(object):

    def __init__(self, name, start_date, end_date, products, created_by, challenge_id=""):
        # type: (str, str, str, str, int, int) -> None

        self.name = name
        self.start_date = start_date
        self.end_date = end_date
        self.products = products
        self.created_by = created_by
        self.challenge_id = challenge_id
