from externals.util import encode_utf_8

from ._Address import Address


class Customer(object):
    def __init__(self, phone, document, name, address):
        # type: (str, str, str, Address) -> None

        self.phone = phone
        self.document = document
        self.name = encode_utf_8(name)
        self.address = address
