from ._OrderPicture import OrderPicture
from ._TablePicture import TablePicture


class Order(object):
    def __init__(
        self,
        store_id,   # type: str
    ):
        # type: (...) -> None

        self.order_picture = OrderPicture(store_id)
        self.table_picture = TablePicture(store_id)

