from abc import ABCMeta, abstractmethod

from typing import Dict


class ImagesServiceModel(object):
    __metaclass__ = ABCMeta

    @abstractmethod
    def get_images(
            self,
            images_path,        # type: str
            totem_config_path   # type: str
    ):
        # type: (...) -> Dict

        raise NotImplementedError
