from abc import ABCMeta, abstractmethod

from ._HttpRequest import HttpRequest
from ._HttpResponse import HttpResponse


class HttpClient(object):
    __metaclass__ = ABCMeta

    @abstractmethod
    def send(self, request):
        # type: (HttpRequest) -> HttpResponse
        raise NotImplementedError()
