class InvalidOrderEventException(RuntimeError):
    def __init__(self, data):
        message = 'Invalid Order Event. Event: "{}"'.format(data)
        super(InvalidOrderEventException, self).__init__(message)
