# -*- coding: utf-8 -*-
from externals.util import encode_utf_8


class Address(object):
    def __init__(self, address_id, zip_code, city, street, neighborhood, number, complement, reference_point):
        # type: (str, str, str, str, str, str, str, str) -> None

        self.address_id = address_id
        self.zip_code = zip_code
        self.city = encode_utf_8(city)
        self.street = encode_utf_8(street)
        self.neighborhood = encode_utf_8(neighborhood)
        self.number = encode_utf_8(number)
        self.complement = encode_utf_8(complement)
        self.reference_point = encode_utf_8(reference_point)
