from httpmessageprocessorutil.automapped import auto_map


@auto_map({"system_api_key": {"name": "X-Api-Key", "location": "header", "type": str},
           "pos_id": {"name": "posId", "type": str},
           "tender_id": {"name": "tenderId", "type": int}})
class CancelPaymentRequest(object):
    def __init__(
            self,
            system_api_key,   # type: str
            pos_id,           # type: str
            tender_id         # type: int
     ):
        # type: (...) -> None

        self.system_api_key = system_api_key
        self.pos_id = pos_id
        self.tender_id = tender_id
