from _StatusResponse import StatusResponse


class GetStoreResponse(object):
    def __init__(
            self,
            status,      # type: bool
            menu_id,     # type: str
            printer,     # type: StatusResponse
            eft,         # type: StatusResponse
            fiscal,      # type: StatusResponse
            manager,     # type: StatusResponse
    ):
        # type: (...) -> None

        self.isOnline = status
        self.menuId = menu_id

        self.printer = printer
        self.eft = eft
        self.fiscal = fiscal
        self.manager = manager


