from helper import ExtendedEnum


class ListenedEvents(ExtendedEnum):

    TABLE_TOTALED = "TABLE_TOTALED"
    ORDER_MODIFIED = "ORDER_MODIFIED"
    ORDER_MODIFIED_VOID = "ORDER_MODIFIED_VOID_ORDER"
    ORDER_MODIFIED_PAID = "ORDER_MODIFIED_PAID"
    ORDER_MODIFIED_TOTAL = "ORDER_MODIFIED_TOTALED"
    ORDER_MODIFIED_VOID_LINE = "ORDER_MODIFIED_VOID_LINE"
    ORDER_MODIFIED_REOPENED = "ORDER_MODIFIED_REOPENED"
    ORDER_MODIFIED_RECALLED = "ORDER_MODIFIED_RECALLED"
