# encoding: utf-8
from _TefCancellationBodyDto import TefCancellationBodyDto
from _TefCancellationHeaderDto import TefCancellationHeaderDto


class TefCancellationGeneratorDto(object):
    def __init__(
        self,
        header,     # type: TefCancellationHeaderDto
        body        # type: TefCancellationBodyDto
    ):
        # type: (...) -> None

        self.header = header
        self.body = body
